module top(
        input               board_clk,
        input               rstn,

        input               i_serial_p,
        input               i_serial_n,
        output              o_serial_p,
        output              o_serial_n,
        
        output              led_d1,
        output              led_d2,
        output              led_d3,
        output              led_d4,

        input               uart_rx,
        output              uart_tx
    );

localparam      BIT_ONE             =   1'b1;
localparam      BIT_ZERO            =   1'b0;
localparam      ZERO_32             =   32'd0;

wire            rstn_ip_rstn;
wire            rstn_ip_uart;
wire            rstn_ip;

wire            pll_lock;
wire            clk_625m_0;
wire            clk_625m_90;
wire            clk_625m_180;
wire            clk_625m_270;
wire            clk_125m;

wire            clk_os_txrx;
wire            reset;
wire            resetn;

wire            pll_uart_lock;
reg     [15:0]  uart_delay_cnt = 16'd0;
reg             rstn_uart_d1 = 0;
reg             rstn_uart_d2 = 0;
wire            clk_uart;
reg             rstn_uart;

//UART PORT
wire    [15:0]  uart_addr;
wire            uart_wren;
wire            uart_wdat_rdy;
wire    [31:0]  uart_wdata;
wire            uart_rden;
wire            uart_rdat_vld;
wire    [31:0]  uart_rdata;

wire    [31:0]  reg0x0000;  wire    [31:0]  reg0x0010;  wire    [31:0]  reg0x0020;  wire    [31:0]  reg0x0030;
wire    [31:0]  reg0x0001;  wire    [31:0]  reg0x0011;  wire    [31:0]  reg0x0021;
wire    [31:0]  reg0x0002;                              wire    [31:0]  reg0x0022;

wire    [31:0]  mon0x0080;  wire    [31:0]  mon0x0090;
wire    [31:0]  mon0x0081;  wire    [31:0]  mon0x0091;
                            wire    [31:0]  mon0x0092;
                            wire    [31:0]  mon0x0093;
//IP configuration_vector
wire            basex_or_sgmii;
wire            sgmii_mode;
wire    [ 8:0]  basex_link_timer;
wire    [ 8:0]  sgmii_link_timer;

wire            main_reset;
wire            an_enable;
wire            an_restart;
wire    [ 2:0]  confi_vector;

reg             ip_link_complete;
reg             an_complete;
reg             an_complete_d1;
reg     [ 7:0]  ip_link_complete_cnt_tx;
reg             tx_mac_rst_n;
reg     [ 7:0]  ip_link_complete_cnt_rx;
reg             rx_mac_rst_n;
reg             auto_negotiation_complete;

reg             cycle_one_time;
reg             cycle_all_time;

//IP TX PORT
wire            tx_mac_clk/* synthesis syn_keep=1 */;
wire            tx_mac_valid;
wire    [ 7:0]  tx_mac_data;
wire            tx_mac_last;
wire            tx_mac_error;
wire            tx_mac_ready;

wire            tx_pause_req;
wire    [15:0]  tx_pause_val;
wire    [47:0]  tx_pause_source_addr;
wire            tx_statistics_valid;

//IP RX PORT
wire            rx_mac_clk/* synthesis syn_keep=1 */;
wire            rx_mac_valid;
wire    [ 7:0]  rx_mac_data;
wire            rx_mac_last;
wire            rx_mac_error;

wire            rx_statistics_valid;
wire    [26:0]  rx_statistics_vector;
wire            rx_pause_req;
wire    [15:0]  rx_pause_val;
//IP PORT
reg     [14:0]  configuration_vector;
wire    [10:0]  status_vector;
wire    [15:0]  link_partner_ability_base;

wire            tx_running;
wire            rx_running;
reg             rx_running_d1;

button sw1(
        .clk   							(board_clk      			),
        .bin   							(rstn           			),
        .bout  							(rstn_ip_rstn   			)
    );

button uart1(
        .clk   							(board_clk      			),
        .bin   							(reg0x0030[0]   			),
        .bout  							(rstn_ip_uart   			)
    );

assign  rstn_ip =   rstn_ip_rstn && rstn_ip_uart;

pll_hclk u_pll_hclk(
        .lock           				(pll_lock           		), //output lock
        .clkout0        				(clk_625m_0         		), //output clkout0
        .clkout1        				(clk_625m_90        		), //output clkout1
        .clkout2        				(clk_625m_180       		), //output clkout2
        .clkout3        				(clk_625m_270       		), //output clkout3
        .clkout4        				(clk_125m           		), //output clkout3
        .reset          				(~rstn_ip           		), //input reset
        .clkin          				(board_clk          		)  //input clkin
    );

pll_uart u_pll_uart(
        .clkout0						(clk_uart      				), //output clkout
        .lock   						(pll_uart_lock 				), //output lock
        .reset  						(~rstn         				), //input reset
        .clkin  						(board_clk     				) //input clkin
    );

always @(posedge clk_uart) begin
    if(~pll_uart_lock) begin
        uart_delay_cnt  <=  16'd0;
    end else begin
        if(uart_delay_cnt == 16'hffff) begin
            uart_delay_cnt  <=  uart_delay_cnt;
        end else begin
            uart_delay_cnt  <=  uart_delay_cnt + 16'd1;
        end
    end
end

always @(posedge clk_uart) begin
    rstn_uart_d1    <=  uart_delay_cnt[15];
    rstn_uart_d2    <=  rstn_uart_d1;
    rstn_uart       <=  rstn_uart_d2;
end

//1000BASE-X 有MAC层
Giga_Serial_Ethernet_LVDS_Top u_giga_serial_ethernet_lvds_top(
        .RX_P                           (i_serial_p                 ),
        .RX_N                           (i_serial_n                 ),
        .TX_P                           (o_serial_p                 ),
        .TX_N                           (o_serial_n                 ),

		.pll_clkin_i                    (board_clk                  ),
        .pll_clkout0_i                  (clk_625m_0                 ),
        .pll_clkout1_i                  (clk_625m_90                ),
        .pll_clkout2_i                  (clk_625m_180               ),
        .pll_clkout3_i                  (clk_625m_270               ),
        .pll_clkout4_i                  (clk_125m                   ),
        .pll_lock_i                     (pll_lock                   ),
        .share_clk0_o                   (                           ),
        .share_clk1_o                   (                           ),
        .share_clk2_o                   (                           ),
        .share_clk3_o                   (                           ),
        .share_clk4_o                   (                 			),
        .share_clk5_o                   (                           ),
        .share_reset_o                  (reset                      ),
		
		.pcs_rstn_i                     (BIT_ONE                    ),

        .prbs_enable_i                  (BIT_ZERO                   ),
        .prbs_err_o                     (                           ),
        .prbs_lock_o                    (                           ),
        .prbs_err_num_o                 (                           ),

        .signal_detect_i                (BIT_ONE                    ), //1:presence; 0:not presence
    //config port
    //link_timer, multiplied by 32.768us
        .basex_link_timer_i             (basex_link_timer           ),
        .sgmii_link_timer_i             (sgmii_link_timer           ),
        .basex_or_sgmii_i               (basex_or_sgmii             ), // 0:base-x; 1:sgmii
        .sgmii_mode_i                   (sgmii_mode                 ), // 0:mac mode; 1:phy mode
        .repeater_mode_i                (BIT_ZERO                   ),

        .rx_mac_clk_o                   (rx_mac_clk                 ),
        .rx_mac_valid_o                 (rx_mac_valid               ),
        .rx_mac_data_o                  (rx_mac_data                ),
        .rx_mac_last_o                  (rx_mac_last                ),
        .rx_mac_error_o                 (rx_mac_error               ),
    
        .rx_fcs_fwd_ena_i               (1'b0                       ),
        .rx_jumbo_ena_i                 (1'b0                       ),
        .rx_statistics_valid_o          (rx_statistics_valid        ),
        .rx_statistics_vector_o         (rx_statistics_vector       ),
        .rx_pause_req_o                 (rx_pause_req               ),
        .rx_pause_val_o                 (rx_pause_val               ),
    
        .tx_mac_clk_o                   (tx_mac_clk                 ),
        .tx_mac_valid_i                 (tx_mac_valid               ),
        .tx_mac_data_i                  (tx_mac_data                ),
        .tx_mac_last_i                  (tx_mac_last                ),
        .tx_mac_error_i                 (tx_mac_error               ),
        .tx_mac_ready_o                 (tx_mac_ready               ),
        .tx_collision_o                 (                           ),
        .tx_retransmit_o                (                           ),

        .tx_pause_req_i                 (tx_pause_req               ),
        .tx_pause_val_i                 (tx_pause_val               ),
        .tx_pause_source_addr_i         (tx_pause_source_addr       ),
        .tx_ifg_delay_ena_i             (1'b0                       ),
        .tx_ifg_delay_i                 (8'd0                       ),
        .tx_fcs_fwd_ena_i               (1'b0                       ),
        .tx_statistics_valid_o          (tx_statistics_valid        ),
        .tx_statistics_vector_o         (                           ),    
    //mdio
        .miim_hs_clk_i                  (1'd0                       ), // input clk
        .miim_rstn_i                    (1'd0                       ), // input rstn
        .miim_phyad_i                   (5'd0                       ), // input [4:0] miim_phyad
        .miim_regad_i                   (5'd0                       ), // input [4:0] miim_regad
        .miim_wrdata_i                  (16'd0                      ), // input [15:0] miim_wrdata
        .miim_wren_i                    (1'd0                       ), // input miim_wren
        .miim_rden_i                    (1'd0                       ), // input miim_rden
        .miim_rddata_o                  (                           ), // output [15:0] miim_rddata
        .miim_rddata_valid_o            (                           ), // output miim_rddata_valid
        .miim_busy_o                    (                           ), // output miim_busy
        .mdc_o                          (                           ), // output mdc
        .mdio_in_i                      (1'd0                       ), // input mdio_in
        .mdio_out_o                     (                           ), // output mdio_out
        .mdio_oen_o                     (                           ), // output mdio_oen
    //pcs config
        .configuration_vector_i         (configuration_vector       ),
        .status_vector_o                (status_vector              ),
        .link_partner_ability_base_o    (link_partner_ability_base  )
    );

assign  resetn          =   ~reset;

assign  main_reset      =   reg0x0010[0];
assign  an_enable       =   reg0x0011[1];
assign  an_restart      =   reg0x0011[0];
assign  basex_or_sgmii  =   reg0x0020[0];
assign  sgmii_mode      =   reg0x0020[1];
assign  confi_vector    =   reg0x0020[6:4];
assign  basex_link_timer=   reg0x0021[8:0];
assign  sgmii_link_timer=   reg0x0022[8:0];

always @(posedge rx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        configuration_vector          <=  {3'b011,4'b0011,4'b1000,4'b0100};
    end else begin
        configuration_vector[0]       <=  main_reset;
        configuration_vector[2]       <=  an_enable;
        configuration_vector[4]       <=  an_restart;
        configuration_vector[1]       <=  configuration_vector[1];
        configuration_vector[3]       <=  configuration_vector[3];
        configuration_vector[10:5]    <=  configuration_vector[10:5];
        configuration_vector[13:11]   <=  confi_vector;
        configuration_vector[14]      <=  configuration_vector[14];
    end
end

always @(posedge rx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        ip_link_complete    <=  1'b0;
    end else if(an_enable) begin
        ip_link_complete    <=   status_vector[2] && status_vector[1] && status_vector[0];
    end else begin
        ip_link_complete    <=   status_vector[1] && status_vector[0];    
    end
end

always @(posedge rx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        an_complete     <=  1'b0;
        an_complete_d1  <=  1'b0;
    end else begin
        an_complete     <=  status_vector[2];
        an_complete_d1  <=  an_complete;
    end
end

always @(posedge rx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        ip_link_complete_cnt_rx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_rx == 8'h7f) begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx;
        end else begin
            ip_link_complete_cnt_rx <=  ip_link_complete_cnt_rx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_rx <=  8'd0;
    end
end

always @(posedge rx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        rx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_rx == 8'h7f) ) begin
        rx_mac_rst_n    <=  1'b1;
    end else begin
        rx_mac_rst_n    <=  1'b0;  
    end
end

always @(posedge tx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        ip_link_complete_cnt_tx <=  8'd0;
    end else if(ip_link_complete)begin
        if(ip_link_complete_cnt_tx == 8'h7f) begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx;
        end else begin
            ip_link_complete_cnt_tx <=  ip_link_complete_cnt_tx + 8'd1;
        end
    end else begin
        ip_link_complete_cnt_tx <=  8'd0;
    end
end

always @(posedge tx_mac_clk or negedge resetn) begin
    if(!resetn) begin
        tx_mac_rst_n    <=  1'b0;
    end else if(ip_link_complete && (ip_link_complete_cnt_tx == 8'h7f) ) begin
        tx_mac_rst_n    <=  1'b1;
    end else begin
        tx_mac_rst_n    <=  1'b0;  
    end
end

always @(posedge rx_mac_clk or negedge resetn) begin
    if(~resetn) begin
        auto_negotiation_complete   <=  1'b0;
    end else if(reg0x0002[0]) begin
        auto_negotiation_complete   <=  1'b0;
    end else if(auto_negotiation_complete) begin
        auto_negotiation_complete   <=  1'b1;
    end else begin
        if(an_complete && ~an_complete_d1) begin
            auto_negotiation_complete   <=  1'b1;
        end else begin
            auto_negotiation_complete   <=  1'b0;
        end
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        rx_running_d1   <=  1'b0;
    end else begin
        rx_running_d1   <=  rx_running;
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        cycle_one_time  <=  1'b0;
    end else begin
        cycle_one_time  <=  reg0x0030[4];
    end
end

always @(posedge rx_mac_clk or negedge rx_mac_rst_n) begin
    if(~rx_mac_rst_n) begin
        cycle_all_time  <=  1'b0;
    end else if(reg0x0030[5]) begin
        cycle_all_time  <=  1'b1;
    end else begin
        if(cycle_all_time) begin
            if(rx_running && ~rx_running_d1) begin
                cycle_all_time  <=  1'b0;
            end else begin
                cycle_all_time  <=  1'b1;
            end
        end else begin
            cycle_all_time  <=  1'b0;
        end
    end
end

Uart_to_Bus_Top u_Uart_to_Bus_Top(
        .rst_n_i                        (rstn_uart                  ),
        .clk_i                          (clk_uart                   ),
        .local0_wren_o                  (uart_wren                  ),
        .local0_addr_o                  (uart_addr                  ),
        .local0_rden_o                  (uart_rden                  ),
        .local0_wdat_o                  (uart_wdata                 ),
        .local0_rdat_i                  (uart_rdata                 ),
        .local0_rdat_vld_i              (uart_rdat_vld              ),
        .local0_wdat_rdy_i              (uart_wdat_rdy              ),
        .uart_rx_led_o                  (                           ),
        .uart_tx_led_o                  (                           ),
        .uart_rx_i                      (uart_rx                    ),
        .uart_tx_o                      (uart_tx                    )
    );

sysreg u_sysreg(                           
        .i_rst_n                        (rstn_uart                  ), 
        .i_clk                          (clk_uart                   ),
                                                    
        .uart_wren                      (uart_wren                  ),
        .uart_addr                      (uart_addr                  ),
        .uart_rden                      (uart_rden                  ),
        .uart_wdata                     (uart_wdata                 ),
        .uart_rdata                     (uart_rdata                 ),
        .uart_rdat_vld                  (uart_rdat_vld              ),
        .uart_wdat_rdy                  (uart_wdat_rdy              ),

        .reg0x0000(reg0x0000),.reg0x0010(reg0x0010),.reg0x0020(reg0x0020),.reg0x0030(reg0x0030),
        .reg0x0001(reg0x0001),.reg0x0011(reg0x0011),.reg0x0021(reg0x0021),.reg0x0031(         ),
        .reg0x0002(reg0x0002),.reg0x0012(         ),.reg0x0022(reg0x0022),.reg0x0032(         ),
        .reg0x0003(         ),.reg0x0013(         ),.reg0x0023(         ),.reg0x0033(         ),

        .mon0x0080(mon0x0080),.mon0x0090(mon0x0090),.mon0x00A0(ZERO_32  ),.mon0x00B0(ZERO_32  ),
        .mon0x0081(mon0x0081),.mon0x0091(mon0x0091),.mon0x00A1(ZERO_32  ),.mon0x00B1(ZERO_32  ),
        .mon0x0082(ZERO_32  ),.mon0x0092(mon0x0092),.mon0x00A2(ZERO_32  ),.mon0x00B2(ZERO_32  ),
        .mon0x0083(ZERO_32  ),.mon0x0093(mon0x0093),.mon0x00A3(ZERO_32  ),.mon0x00B3(ZERO_32  )
    );

assign	mon0x0080[30:3] 	= 28'd0;
assign	mon0x0081[31:3] 	= 29'd0;
assign	mon0x0090[31:12]	= 20'd0;
assign	mon0x0091[31:2]		= 30'd0;

assign  mon0x0080[31]  		=   auto_negotiation_complete;
assign  mon0x0081[2:0] 		=   status_vector[7:5];

assign  mon0x0090[11:0]		= { 3'd0                                            ,ip_link_complete   ,
                       		    rx_mac_rst_n    ,tx_mac_rst_n   ,rstn_ip        ,rstn_uart          ,
                       		    resetn          ,1'b0           ,pll_uart_lock  ,pll_lock           };

mac_tx_model u_mac_tx_model(
        .tx_mac_clk               		(tx_mac_clk                 ),
        .tx_mac_rst_n             		(tx_mac_rst_n               ),
        
    //CONTROL PORT
        .i_cycle_one_time         		(cycle_one_time             ),
        .i_cycle_all_time         		(cycle_all_time             ),

        .i_tx_length_hold         		(reg0x0000[0]               ),
        .i_tx_no_pause            		(reg0x0001[0]               ),
        .i_tx_check_clear         		(reg0x0002[0]               ),

        .tx_running               		(tx_running                 ),

    //IP TX PORT
        .tx_mac_valid             		(tx_mac_valid               ),
        .tx_mac_data              		(tx_mac_data                ),
        .tx_mac_last              		(tx_mac_last                ),
        .tx_mac_error             		(tx_mac_error               ),
        .tx_mac_ready             		(tx_mac_ready               ),

        .tx_pause_req             		(tx_pause_req               ),
        .tx_pause_val             		(tx_pause_val               ),
        .tx_pause_source_addr     		(tx_pause_source_addr       ),
        .tx_statistics_valid      		(tx_statistics_valid        ),
        .tx_cnt_er                		(mon0x0091[0]               ),
        .tx_cnt                   		(mon0x0092                  )
    );

mac_rx_model u_mac_rx_model(
        .rx_mac_clk               		(rx_mac_clk                 ),
        .rx_mac_rst_n             		(rx_mac_rst_n               ),
                
    //CONTROL PORT
        .i_cycle_one_time         		(cycle_one_time             ),
        .i_cycle_all_time         		(cycle_all_time             ),

        .i_rx_length_hold         		(reg0x0000[0]               ),
        .i_rx_no_pause            		(reg0x0001[0]               ),
        .i_rx_check_clear         		(reg0x0002[0]               ),

        .rx_running               		(rx_running                 ),
    //IP TX PORT
        .rx_mac_valid             		(rx_mac_valid               ),
        .rx_mac_data              		(rx_mac_data                ),
        .rx_mac_last              		(rx_mac_last                ),
        .rx_mac_error             		(rx_mac_error               ),

        .rx_statistics_valid      		(rx_statistics_valid        ),
        .rx_statistics_vector     		(rx_statistics_vector       ),
        .rx_pause_req             		(rx_pause_req               ),
        .rx_pause_val             		(rx_pause_val               ),

    //CHECK RESULT
        .rx_data_error            		(                           ),
        .rx_cnt_error             		(                           ),
        .rx_statistics_error      		(                           ),
        .rx_error_all_data        		(                           ),

        .rx_data_error_pause      		(                           ),
        .rx_length_error_pause    		(                           ),
        .rx_statistics_error_pause		(                           ),
        .rx_pause_error           		(                           ),
        .rx_error_all_pause       		(                           ),

        .rx_error_all             		(                           ),
        .rx_error_all_data_check  		(mon0x0080[0]               ),
        .rx_error_all_pause_check 		(mon0x0080[1]               ),
        .rx_error_all_check       		(mon0x0080[2]               ),
        .rx_cnt_er                		(mon0x0091[1]               ),
        .rx_cnt                   		(mon0x0093                  )
    );

assign  led_d1 = ip_link_complete;
assign  led_d2 = tx_running || rx_running;
assign  led_d3 = mon0x0080[0];
assign  led_d4 = mon0x0080[1];

endmodule